@@ -2,7 +2,7 @@
 //
 // Catalog.h
 //
-// Copyright 1996-2003 Glyph & Cog, LLC
+// Copyright 1996-2007 Glyph & Cog, LLC
 //
 //========================================================================
 
@@ -19,8 +19,8 @@ class Page;
 class PageAttrs;
 struct Ref;
 class LinkDest;
-class UGooString;
 class PageLabelInfo;
+class Form;
 
 //------------------------------------------------------------------------
 // NameTree
@@ -31,18 +31,18 @@ public:
   NameTree();
   void init(XRef *xref, Object *tree);
   void parse(Object *tree);
-  GBool lookup(UGooString *name, Object *obj);
+  GBool lookup(GooString *name, Object *obj);
   void free();
   int numEntries() { return length; };
   // iterator accessor
   Object getValue(int i);
-  UGooString *getName(int i);
+  GooString *getName(int i);
 
 private:
   struct Entry {
     Entry(Array *array, int index);
     ~Entry();
-    UGooString *name;
+    GooString name;
     Object value;
     void free();
     static int cmp(const void *key, const void *entry);
@@ -61,12 +61,16 @@ private:
 class EmbFile {
 public:
   EmbFile(GooString *name, GooString *description, 
+	  int size,
 	  GooString *createDate,
-	  GooString *modDate, Object objStr) :
+	  GooString *modDate, GooString *checksum,
+	  Object objStr) :
     m_name(name),
     m_description(description),
+    m_size(size),
     m_createDate(createDate),
-    m_modDate(modDate)
+    m_modDate(modDate),
+    m_checksum(checksum)
   {
     objStr.copy(&m_objStr);
   }
@@ -77,20 +81,25 @@ public:
     delete m_description;
     delete m_modDate;
     delete m_createDate;
+    delete m_checksum;
     m_objStr.free();
   }
 
   GooString *name() { return m_name; }
   GooString *description() { return m_description; }
+  int size() { return m_size; }
   GooString *modDate() { return m_modDate; }
   GooString *createDate() { return m_createDate; }
+  GooString *checksum() { return m_checksum; }
   Object &streamObject() { return m_objStr; }
 
 private:
   GooString *m_name;
   GooString *m_description;
+  int m_size;
   GooString *m_createDate;
   GooString *m_modDate;
+  GooString *m_checksum;
   Object m_objStr;
 };
 
@@ -135,7 +144,9 @@ public:
 
   // Find a named destination.  Returns the link destination, or
   // NULL if <name> is not a destination.
-  LinkDest *findDest(UGooString *name);
+  LinkDest *findDest(GooString *name);
+
+  Object *getDests() { return &dests; }
 
   // Get the number of embedded files
   int numEmbeddedFiles() { return embeddedFileNameTree.numEntries(); }
@@ -151,6 +162,8 @@ public:
 
   Object *getAcroForm() { return &acroForm; }
 
+  Form* getForm() { return form; }
+
   enum PageMode {
     pageModeNone,
     pageModeOutlines,
@@ -166,7 +179,7 @@ public:
     pageLayoutTwoColumnLeft,
     pageLayoutTwoColumnRight,
     pageLayoutTwoPageLeft,
-    pageLayoutTwoPageRight,
+    pageLayoutTwoPageRight
   };
 
   // Returns the page mode.
@@ -178,6 +191,7 @@ private:
   XRef *xref;			// the xref table for this PDF file
   Page **pages;			// array of pages
   Ref *pageRefs;		// object ID for each page
+  Form *form;
   int numPages;			// number of pages
   int pagesSize;		// size of pages array
   Object dests;			// named destination dictionary
@@ -193,7 +207,8 @@ private:
   PageMode pageMode;		// page mode
   PageLayout pageLayout;	// page layout
 
-  int readPageTree(Dict *pages, PageAttrs *attrs, int start, int callDepth);
+  int readPageTree(Dict *pages, PageAttrs *attrs, int start,
+		   char *alreadyRead);
   Object *findDestInTree(Object *tree, GooString *name, Object *obj);
 };
 
